module UniversalGitClient
  module Errors
    class HttpError < StandardError
      attr_reader :response

      def initialize(msg, response)
        @response = response
        super(msg)
      end
    end

    class ClientError < HttpError; end
    class ServerError < HttpError; end

    class BadRequest < ClientError; end
    class Unauthorized < ClientError; end
    class Forbidden < ClientError; end
    class NotFound < ClientError; end
    class MethodNotAllowed < ClientError; end
    class NotAcceptable < ClientError; end
    class Conflict < ClientError; end
    class UnsupportedMediaType < ClientError; end
    class UnprocessableEntity < ClientError; end

    class InternalServerError < ServerError; end
    class BadGateway < ServerError; end
    class ServiceUnavailable < ServerError; end
  end
end
