require 'universal-git-client/http/base'
require 'universal-git-client/http/helpers/dummy_response'

module UniversalGitClient
  module Http
    class BitbucketServer < Base
      def user
        Helpers::DummyResponse.response(options: default_options)
      end

      def organizations(page: 1, per_page: nil)
        options = default_options.merge(
          query: {
            start: limit(per_page) * (page - 1),
            limit: limit(per_page)
          }
        )
        body = {
          size: 0,
          start: 0,
          limit: limit(per_page),
          values: []
        }
        Helpers::DummyResponse.response(body: body, options: options)
      end

      def user_repos(page: 1, per_page: nil)
        with_response_validation! do
          self.class.get(
            '/repos',
            default_options.merge(
              query: {
                state: 'AVAILABLE',
                start: limit(per_page) * (page - 1),
                limit: limit(per_page)
              }
            )
          )
        end
      end

      def orga_repos(organization:, page: 1, per_page: nil)
        options =  default_options.merge(
          query: {
            start: limit(per_page) * (page - 1),
            limit: limit(per_page)
          }
        )
        body = {
          size: 0,
          start: 0,
          limit: limit(per_page),
          values: []
        }
        Helpers::DummyResponse.response(body: body, options: options)
      end

      def repository(owner:, repo:)
        with_response_validation! do
          self.class.get("/projects/#{owner}/repos/#{repo}", default_options)
        end
      end

      def branches(owner:, repo:, page: 1, per_page: nil)
        with_response_validation! do
          self.class.get(
            "/projects/#{owner}/repos/#{repo}/branches",
            default_options.merge(
              query: {
                start: limit(per_page) * (page - 1),
                limit: limit(per_page)
              }
            )
          )
        end
      end

      def branch(owner:, repo:, branch:)
        with_response_validation! do
          response = self.class.get(
            "/projects/#{owner}/repos/#{repo}/branches",
            default_options.merge(
              query: {
                filterText: branch,
                limit: 1
              }
            )
          )
          Helpers::DummyResponse.response(request: response.request, body: response.parsed_response['values'].first)
        end
      end

      def download_repo_archive(owner:, repo:, branch: nil)
        Down.download("#{base_url}/projects/#{owner}/repos/#{repo}/archive?at=#{branch}&format=zip", down_default_options)
      end

      def setup_repo_webhook(owner:, repo:, webhook_url:, webhook_secret: nil)
        with_response_validation! do
          self.class.post(
            "/projects/#{owner}/repos/#{repo}/webhooks",
            default_options.merge(
              body: {
                name: 'webhook',
                url: webhook_url,
                active: true,
                events: [
                  'repo:refs_changed'
                ],
                configuration: {
                  secret: webhook_secret
                }
              }.to_json
            )
          )
        end
      end

      def delete_repo_webhook(owner:, repo:, webhook_id:)
        with_response_validation! do
          self.class.delete(
            "/projects/#{owner}/repos/#{repo}/webhooks/#{webhook_id}",
            default_options
          )
        end
      end

      private

      def limit(per_page)
        per_page || default_elements_per_page
      end
    end
  end
end
