require 'universal-git-client/http/base'

module UniversalGitClient
  module Http
    class Github < Base
      def user
        with_response_validation! do
          self.class.get('/user', default_options)
        end
      end

      def organizations(page: 1, per_page: nil)
        with_response_validation! do
          self.class.get(
            '/user/orgs',
            default_options.merge(
              query: {
                page: page,
                per_page: per_page || default_elements_per_page,
              },
            )
          )
        end
      end

      def user_repos(page: 1, per_page: nil)
        with_response_validation! do
          self.class.get(
            '/user/repos',
            default_options.merge(
              query: {
                affiliation: 'owner',
                page: page,
                per_page: per_page || default_elements_per_page,
              },
            )
          )
        end
      end

      def orga_repos(organization:, page: 1, per_page: nil)
        # TODO: Handle permissions
        with_response_validation! do
          self.class.get(
            "/orgs/#{organization}/repos",
            default_options.merge(
              query: {
                page: page,
                per_page: per_page || default_elements_per_page,
              },
            )
          )
        end
      end

      def repository(owner:, repo:)
        with_response_validation! do
          self.class.get("/repos/#{owner}/#{repo}", default_options)
        end
      end

      def branches(owner:, repo:, page: 1, per_page: nil)
        with_response_validation! do
          self.class.get(
            "/repos/#{owner}/#{repo}/branches",
            default_options.merge(
              query: {
                page: page,
                per_page: per_page || default_elements_per_page,
              },
            )
          )
        end
      end

      def branch(owner:, repo:, branch:)
        encoded_branch = URI.encode_www_form_component(branch)
        with_response_validation! do
          self.class.get(
            "/repos/#{owner}/#{repo}/branches/#{encoded_branch}",
            default_options
          )
        end
      end

      def download_repo_archive(owner:, repo:, branch: nil)
        path = "#{base_url}/repos/#{owner}/#{repo}/zipball"
        path << "/#{branch}" if branch
        Down.download(path, down_default_options)
      end

      def setup_repo_webhook(owner:, repo:, webhook_url:, webhook_secret: nil)
        with_response_validation! do
          self.class.post(
            "/repos/#{owner}/#{repo}/hooks",
            default_options.merge(
              body: {
                name: 'web',
                active: true,
                events: [
                  'push',
                ],
                config: {
                  url: webhook_url,
                  secret: webhook_secret,
                  content_type: 'json',
                }.compact,
              }.to_json,
            )
          )
        end
      end

      def delete_repo_webhook(owner:, repo:, webhook_id:)
        with_response_validation! do
          self.class.delete(
            "/repos/#{owner}/#{repo}/hooks/#{webhook_id}",
            default_options
          )
        end
      end
    end
  end
end
