require 'universal-git-client/normalizers/base'

module UniversalGitClient
  module Normalizers
    class BitbucketServer < Base
      def current_page_index
        current_page.to_s
      rescue StandardError
        nil
      end

      def first_page_index
        '1'
      end

      def prev_page_index
        prev_page&.to_s
      end

      def next_page_index
        next_page&.to_s
      end

      def last_page_index
        nil
      end

      def per_page_index
        response.request.options[:query][:limit].to_s
      rescue StandardError
        nil
      end

      private

      def next_page
        next_page_start = response.parsed_response['nextPageStart']
        if next_page_start
          ( next_page_start.to_i / per_page_index.to_i) + 1
        else
          nil
        end
      rescue StandardError
        nil
      end

      def current_page
        (response.request.options[:query][:start].to_i / per_page_index.to_i) + 1
      end

      def prev_page
        current_page > 1 ? current_page - 1 : nil
      end

      class UserSerializer < BaseSerializer
        set_id :account_id
        attribute :login, &:username
        attribute :name, &:display_name

        attribute :avatar_url do |object|
          object.links.avatar.href
        end
      end

      class OrganizationSerializer < BaseSerializer
        set_id :uuid
        attribute :login, &:username

        attribute :avatar_url do |object|
          object.links.avatar.href
        end
      end

      class RepositorySerializer < BaseSerializer
        attribute :name, &:slug
        attribute :archived do false end
        attribute :default_branch do nil end

        attribute :full_name do |object|
          "#{object.project.key}/#{object.name}"
        end

        attribute :full_path do |object|
          "#{object.project.key}/#{object.slug}"
        end

        attribute :private do |object|
          !object.public
        end
      end

      class BranchSerializer < BaseSerializer
        set_id :displayId
        attribute :name, &:displayId
        attribute :commit, &:latestCommit
        attribute :protected do false end
      end

      class WebhookSerializer < BaseSerializer
        set_id :id
        attributes :url, :active
        attribute :events do |object|
          events = []
          events << 'push' if object.events.include?('repo:refs_changed')
        end
        attribute :type do 'Repository' end
      end
    end
  end
end
